from requests import Response


class BadRequest(Exception):
    def __init__(self, msg=None, resp=None):
        self.message = msg
        self.response: Response | None = resp
        super().__init__(self.message)
        
        
class APINotImplemented(Exception):
    def __init__(self):
        super().__init__('This method is not implemented yet')


from pydantic import BaseModel
from pydantic import Field
from typing import Optional, Union, List

class SMSSendSingleMessagePost(BaseModel):
    message: str
    success: bool
    message_id: str


class SMSGetBalance(BaseModel):
    balance: float
    currency: str


class _SMSGetCampaignsData(BaseModel):
    id: str
    type: str
    user_id: int
    sender_id: str
    template_id: Optional[Union[str, int]]
    message: str
    list_id: Optional[Union[str, int]]
    status: str
    client_total_cost: float
    estimated_cost: float
    delivered_messages: int
    failed_messages: int
    pending_messages: int
    sent_messages: int
    bypass_optout: int
    callback_url: str
    scheduled_for: int
    timezone: str
    created_at: str
    updated_at: str
    sms_count: int
    is_api: int
    canceled_at: str
    
class _SMSGetCampaignsLink(BaseModel):
    url: Optional[str]
    label: str
    active: bool
    

class SMSGetCampaigns(BaseModel):
    success: bool
    data: List[_SMSGetCampaignsData]
    links: List[_SMSGetCampaignsLink]
    current_page: int
    from_: int = Field(alias='from')
    last_page: int
    path: str
    per_page: int
    to: int
    total: int
    


__ALL__ = ()

from requests import post, get

from ..exceptions import BadRequest, APINotImplemented
from .models import SMSSendSingleMessagePost, SMSGetBalance, SMSGetCampaigns


class SMS:
    def __init__(self, token):
        self._token = token
        self._basic_headers = {'Authorization': f'Bearer {self._token}',
                               'Content-Type': 'application/json'}

    def send_single_message(self,
                    message: str,
                    to: str,
                    bypass_optout: bool = None,
                    sender_id: str = None,
                    callback_url: str = None
                    ) -> models.SMSSendSingleMessagePost:
        """Send single message to a number

        Args:
            message (str): Your message
            to (str): Phone number
            bypass_optout (bool, optional): True will bypass optouts. Defaults to None.
            sender_id (str, optional): The displayed value of who sent the message. Defaults to None.
            callback_url (str, optional): A callback URL that will be used to send back information about updates of message status. Defaults to None.

        Raises:
            BadRequest

        Returns:
            models.SMSSendSingleMessagePost
        """
        
        
        URL = 'https://api.sms.to/sms/send'
        r = post(URL,
                 json={
                     'message': message,
                     'to': to,
                     'bypass_optout': bypass_optout if bypass_optout is not None else None,
                     'sender_id': sender_id if sender_id is not None else None,
                     'callback_url': callback_url if callback_url is not None else None,
                 },
                 headers=self._basic_headers)
        if r.status_code == 400:
            raise BadRequest(r.json()['message'], r)

        return SMSSendSingleMessagePost.model_validate(r.json())

    def get_balance(self) -> SMSGetBalance:
        """Get the current balance of your SMS.to account

        Raises:
            BadRequest

        Returns:
            SMSGetBalance
        """
        
        
        URL = 'https://auth.sms.to/api/balance'
        r = get(URL, headers=self._basic_headers)
        if r.status_code == 400:
            raise BadRequest(r.json()['message'], r)
        return SMSGetBalance.model_validate(r.json())
    
    def get_messages(limit: int = 15, 
                     order_direction: str = 'desc', 
                     order_by: str = 'created_at', 
                     status: str = None, 
                     to: str = None, 
                     create_at_from: str = None, 
                     create_at_to: str = None):
        URL = 'https://api.sms.to/v2/messages'
        raise APINotImpemented
        
    def get_campaigns(limit: int = 100,
                      page: int = 1,
                      search: str = None) -> SMSGetCampaigns:
        """Fetch paginated campaigns list

        Args:
            limit (int, optional): The number of campaigns per page. Defaults to 100.
            page (int, optional): The page number. Defaults to 1.
            search (str, optional): Keywords to search for. This parameter can be used to filter campaigns by phone number, specific date, source type. Defaults to None.

        Raises:
            BadRequest

        Returns:
            SMSGetCampaigns: _description_
        """
        URL = 'https://api.sms.to/v2/campaigns'
        r = get(URL, params={
            'limit': limit,
            'page': page,
            'search': search if search is not None else None
        })
        if r.status_code == 400:
            raise BadRequest(r.json()['message', r])
        
        return SMSGetCampaigns.model_validate(r.json())
    
    def get_campaign_by_id():
        URL = 'https://api.sms.to/v2/campaigns/[id]'
        raise APINotImplemented
    
    def get_last_campaign():
        URL = 'https://api.sms.to/v2/last/campaign'
        raise APINotImplemented
    
    def get_last_message():
        URL = 'https://api.sms.to/v2/last/message'
        raise APINotImplemented
    
    def get_recent_incoming_sms():
        URL = 'https://sms.to/v1/recent/inbound-sms'
        raise APINotImplemented
    
    def get_message_by_id():
        URL = 'https://api.sms.to/message/[id]'
        raise APINotImplemented
    
    def estimate_personalized_message(messages: list, 
                                      sender_id: str = None):
        URL = 'https://api.sms.to/sms/estimate'
        raise APINotImplemented
    
    def estimate_campaign_message(message: str, 
                                  to: list, 
                                  sender_id: str = None):
        URL = 'https://api.sms.to/sms/estimate'
        raise APINotImplemented
    
    def estimate_single_message(message: str,
                                to: str,
                                sender_id: str = None):
        URL = 'https://api.sms.to/sms/estimate'
        raise APINotImplemented
    
    def send_campaign_message(message: str,
                              to: list,
                              bypass_optout: bool = None,
                              sender_id: str = None,
                              callback_url: str = None):
        URL = 'https://api.sms.to/sms/send'
        raise APINotImplemented
    
    def estimate_list_message(message: str,
                              list_id: str,
                              sender_id: str = None):
        URL = 'https://api.sms.to/sms/estimate'
        raise APINotImplemented
    
    def send_message_to_list(message: str,
                             list_id: str,
                             bypass_optout: bool = None,
                             sender_id: str = None,
                             callback_url: str = None):
        URL = 'https://api.sms.to/sms/send'
        raise APINotImplemented
    
    def send_personalized_messages(messages: list,
                                   sender_id: str = None,
                                   callback_url: str = None):
        URL = 'https://api.sms.to/sms/send'
        raise APINotImplemented
    
    def schedule_sending_messages(messages: list,
                                  sender_id: str = None,
                                  scheduled_for: str = None,
                                  timezone: str = None,
                                  bypass_optout: bool = None):
        URL = 'https://api.sms.to/sms/send'
        raise APINotImplemented
    
    def send_flash_message(message: str,
                           to: str,
                           bypass_optout: bool = None,
                           sender_id: str = None,
                           callback_url: str = None):
        URL = 'https://api.sms.to/fsms/send'
        raise APINotImplemented
    


from setuptools import setup, find_packages

VERSION = '0.0.1' 
DESCRIPTION = 'SMS.to API wrapper'
LONG_DESCRIPTION = 'Manage SMS.to api via Python 3 easily'

# Setting up
setup(
       # the name must match the folder name 'verysimplemodule'
        name="smsto", 
        version=VERSION,
        author="Evgeniy Finskiy",
        author_email="finskiy.ru@yandex.ru",
        description=DESCRIPTION,
        long_description=LONG_DESCRIPTION,
        packages=find_packages(),
        install_requires=['pydantic', 'requests'], 
        
        keywords=['python3', 'smsto', 'sms', 'api'],
        classifiers= [
            "Development Status :: 3 - Alpha",
            "Intended Audience :: Developers",
            "Programming Language :: Python :: 3",
            "Programming Language :: Python :: 3.11",
            "Operating System :: MacOS :: MacOS X",
            "Operating System :: Microsoft :: Windows",
            "Operating System :: Unix",
            "Topic :: Communications",
            "Typing :: Typed"
        ]
)